/*
 * Copyright (C) 2021 Thomas Van Acker
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <https://www.gnu.org/licenses/>.
 */

package be.bitscuit.calendar.Activities;

import androidx.annotation.Nullable;
import androidx.annotation.RequiresApi;
import androidx.appcompat.app.ActionBar;
import androidx.appcompat.app.AppCompatActivity;
import androidx.appcompat.widget.Toolbar;

import android.app.Activity;
import android.app.AlarmManager;
import android.app.AlertDialog;
import android.app.PendingIntent;
import android.app.TimePickerDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.net.Uri;
import android.os.Build;
import android.os.Bundle;
import android.os.ParcelFileDescriptor;
import android.text.Editable;
import android.text.TextWatcher;
import android.view.KeyEvent;
import android.view.MenuItem;
import android.view.View;
import android.view.inputmethod.EditorInfo;
import android.view.inputmethod.InputMethodManager;
import android.widget.Button;
import android.widget.CompoundButton;
import android.widget.EditText;
import android.widget.Switch;
import android.widget.TextView;
import android.widget.TimePicker;
import android.widget.Toast;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.Calendar;
import java.util.Objects;

import be.bitscuit.calendar.BroadcastReceivers.OverviewReceiver;
import be.bitscuit.calendar.BroadcastReceivers.SleepReceiver;
import be.bitscuit.calendar.R;
import be.bitscuit.calendar.Util.TaskData;
import be.bitscuit.calendar.Util.TaskUtil;
import be.bitscuit.calendar.Util.UserPrefs;
import be.bitscuit.calendar.Util.Util;

public class SettingsActivity extends AppCompatActivity {

    private static final int RC_CREATE_FILE = 1;
    private static final int RC_OPEN_FILE = 2;
    
    private TextView labelDeleteAfter, labelOverviewTime, labelSleepTime;
    private Switch switchDelete, switchOverview, switchSleep;
    private EditText editDeleteAfter, editOverviewTime, editSleepTime;
    private Button buttonBackupCreate, buttonBackupLoad;

    private UserPrefs userPrefs;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_settings);

        //Init vars
        userPrefs = new UserPrefs(this);

        //Set toolbar
        Toolbar toolbar = findViewById(R.id.setting_toolbar);
        setSupportActionBar(toolbar);
        setTitle("Settings");
        ActionBar actionBar = getSupportActionBar();
        actionBar.setDisplayHomeAsUpEnabled(true);
        
        //Set views
        switchDelete = findViewById(R.id.settings_switch_delete);
        labelDeleteAfter = findViewById(R.id.settings_label_delete_after);
        editDeleteAfter = findViewById(R.id.settings_edit_delete_after);
        switchOverview = findViewById(R.id.settings_switch_overview);
        labelOverviewTime = findViewById(R.id.settings_label_overview_time);
        editOverviewTime = findViewById(R.id.settings_edit_overview_time);
        switchSleep = findViewById(R.id.settings_switch_sleep);
        labelSleepTime = findViewById(R.id.settings_label_sleep_time);
        editSleepTime = findViewById(R.id.settings_edit_sleep_time);
        buttonBackupCreate = findViewById(R.id.settings_button_backup_create);
        buttonBackupLoad = findViewById(R.id.settings_button_backup_load);

        //Edit view properties
        switchDelete.setOnCheckedChangeListener(new CompoundButton.OnCheckedChangeListener() {
            @Override
            public void onCheckedChanged(CompoundButton compoundButton, boolean b) {
                userPrefs.setAutoDeleteTasks(b);
                updateViews();
            }
        });
        editDeleteAfter.setOnEditorActionListener(new TextView.OnEditorActionListener() {
            @Override
            public boolean onEditorAction(TextView textView, int i, KeyEvent keyEvent) {
                if(i == EditorInfo.IME_ACTION_DONE) {
                    InputMethodManager imm = (InputMethodManager) getSystemService(Context.INPUT_METHOD_SERVICE);
                    imm.hideSoftInputFromWindow(editDeleteAfter.getWindowToken(), 0);
                    editDeleteAfter.clearFocus();
                    return true;
                }
                return false;
            }
        });
        editDeleteAfter.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence charSequence, int i, int i1, int i2) {

            }

            @Override
            public void onTextChanged(CharSequence charSequence, int i, int i1, int i2) {

            }

            @Override
            public void afterTextChanged(Editable editable) {
                //When text changed -> check if valid
                try{
                    int days = Integer.parseInt(editDeleteAfter.getText()+"");
                    if(days < 0){
                        editDeleteAfter.setError("Invalid!");
                        return;
                    }
                    userPrefs.setAutoDeleteTasksAfter(days * 86400000);
                }catch (Exception e){
                    editDeleteAfter.setError("Enter a number!");
                }
            }
        });
        switchOverview.setOnCheckedChangeListener(new CompoundButton.OnCheckedChangeListener() {
            @Override
            public void onCheckedChanged(CompoundButton compoundButton, boolean b) {
                userPrefs.setSendDailyOverview(b);
                updateOverviewAlarm();
                updateViews();
            }
        });
        editOverviewTime.setFocusable(false);
        editOverviewTime.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                long t = userPrefs.getDailyOverviewTime();
                new TimePickerDialog(view.getContext(), new TimePickerDialog.OnTimeSetListener() {
                    @Override
                    public void onTimeSet(TimePicker timePicker, int i, int i1) {
                        userPrefs.setDailyOverviewTime(i*3600000 + i1*60000);

                        //Set alarm
                        updateOverviewAlarm();

                        //Update text
                        updateViews();
                    }
                }, (int)t/3600000, (int)(t%3600000)/60000, true)
                        .show();
            }
        });
        switchSleep.setOnCheckedChangeListener(new CompoundButton.OnCheckedChangeListener() {
            @Override
            public void onCheckedChanged(CompoundButton compoundButton, boolean b) {
                userPrefs.setSendSleepReminder(b);
                updateSleepAlarm();
                updateViews();
            }
        });
        editSleepTime.setFocusable(false);
        editSleepTime.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                long t = userPrefs.getSleepTime();
                new TimePickerDialog(view.getContext(), new TimePickerDialog.OnTimeSetListener() {
                    @Override
                    public void onTimeSet(TimePicker timePicker, int i, int i1) {
                        userPrefs.setSleepTime(i*3600000 + i1*60000);

                        //Set alarm
                        updateSleepAlarm();

                        //Update text
                        updateViews();
                    }
                }, (int)t/3600000, (int)(t%3600000)/60000, true)
                        .show();
            }
        });
        buttonBackupCreate.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                //Open file selector
                if (android.os.Build.VERSION.SDK_INT < android.os.Build.VERSION_CODES.KITKAT) {
                    Toast.makeText(view.getContext(), "Action not supported on API < 19", Toast.LENGTH_SHORT).show();
                    return;
                }
                Intent intent = new Intent(Intent.ACTION_CREATE_DOCUMENT);
                intent.addCategory(Intent.CATEGORY_OPENABLE);
                intent.setType("application/clndr");
                intent.putExtra(Intent.EXTRA_TITLE, "calendar_backup_"+Util.formatMillis(System.currentTimeMillis(), "yyyyMMddHHmmss")+".clndr");

                startActivityForResult(intent, RC_CREATE_FILE);//Result handled in onActivityResult()
            }
        });
        buttonBackupLoad.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                //Open file selector
                if (android.os.Build.VERSION.SDK_INT < android.os.Build.VERSION_CODES.KITKAT) {
                    Toast.makeText(view.getContext(), "Action not supported on API < 19", Toast.LENGTH_SHORT).show();
                    return;
                }
                Intent intent = new Intent(Intent.ACTION_OPEN_DOCUMENT);
                intent.addCategory(Intent.CATEGORY_OPENABLE);
                intent.setType("application/clndr");

                startActivityForResult(intent, RC_OPEN_FILE);
            }
        });

        //Set view data
        updateViews();
    }


    public void updateViews(){
        switchDelete.setChecked(userPrefs.getAutoDeleteTasks());
        editDeleteAfter.setText(""+userPrefs.getAutoDeleteTasksAfter()/86400000);
        labelDeleteAfter.setEnabled(switchDelete.isChecked());
        editDeleteAfter.setEnabled(switchDelete.isChecked());
        switchOverview.setChecked(userPrefs.getSendDailyOverview());
        editOverviewTime.setText(Util.formatMillis(Util.millisToDateMillis(System.currentTimeMillis()) + userPrefs.getDailyOverviewTime(), "HH:mm"));
        labelOverviewTime.setEnabled(switchOverview.isChecked());
        editOverviewTime.setEnabled(switchOverview.isChecked());
        switchSleep.setChecked(userPrefs.getSendSleepReminder());
        editSleepTime.setText(Util.formatMillis(Util.millisToDateMillis(System.currentTimeMillis()) + userPrefs.getSleepTime(), "HH:mm"));
        labelSleepTime.setEnabled(switchSleep.isChecked());
        editSleepTime.setEnabled(switchSleep.isChecked());
    }


    public void updateOverviewAlarm(){//Also change in BootReceiver!!!
        //Delete old alarm
        AlarmManager alarmManager = (AlarmManager) getSystemService(ALARM_SERVICE);
        Intent intent = new Intent(getApplicationContext(), OverviewReceiver.class);
        PendingIntent pendingIntent = PendingIntent.getBroadcast(getApplicationContext(), 0, intent, PendingIntent.FLAG_UPDATE_CURRENT);
        pendingIntent.cancel();
        alarmManager.cancel(pendingIntent);

        //Set new alarm
        if(userPrefs.getSendDailyOverview()) {
            intent = new Intent(getApplicationContext(), OverviewReceiver.class);
            pendingIntent = PendingIntent.getBroadcast(getApplicationContext(), 0, intent, 0);
            long time = Util.millisToDateMillis(System.currentTimeMillis()) + userPrefs.getDailyOverviewTime();
            if (time < System.currentTimeMillis())
                time += 86400000; //When next overview is tomorrow
            alarmManager.setRepeating(AlarmManager.RTC_WAKEUP, time, 86400000, pendingIntent);
        }
    }

    public void updateSleepAlarm(){//Also change in BootReceiver!!!
        //Delete old alarm
        AlarmManager alarmManager = (AlarmManager) getSystemService(ALARM_SERVICE);
        Intent intent = new Intent(getApplicationContext(), SleepReceiver.class);
        PendingIntent pendingIntent = PendingIntent.getBroadcast(getApplicationContext(), 0, intent, PendingIntent.FLAG_UPDATE_CURRENT);
        pendingIntent.cancel();
        alarmManager.cancel(pendingIntent);

        //Set new alarm
        if(userPrefs.getSendSleepReminder()) {
            intent = new Intent(getApplicationContext(), SleepReceiver.class);
            pendingIntent = PendingIntent.getBroadcast(getApplicationContext(), 0, intent, 0);
            long time = Util.millisToDateMillis(System.currentTimeMillis()) + userPrefs.getSleepTime();
            if (time < System.currentTimeMillis())
                time += 86400000; //When next overview is tomorrow
            alarmManager.setRepeating(AlarmManager.RTC_WAKEUP, time, 86400000, pendingIntent);
        }
    }


    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        switch (item.getItemId()) {
            case android.R.id.home:
                onBackPressed();
                return true;

            default:
                return super.onOptionsItemSelected(item);
        }
    }


    @Override
    protected void onActivityResult(int requestCode, int resultCode, @Nullable Intent data) {
        if (requestCode == RC_CREATE_FILE && resultCode == Activity.RESULT_OK) {
            Uri uri = null;
            if (data != null) {
                uri = data.getData();

                //Open file and write to it
                try{
                    TaskUtil taskUtil = new TaskUtil(this);
                    String text = taskUtil.getBackup();
                    writeDocument(uri, text);
                    Toast.makeText(this, "Created backup file!", Toast.LENGTH_SHORT).show();
                }catch (Exception e){
                    Toast.makeText(this, "Error writing file!", Toast.LENGTH_SHORT).show();
                }
            }
        }
        if (requestCode == RC_OPEN_FILE && resultCode == Activity.RESULT_OK) {
            Uri uri = null;
            if (data != null) {
                uri = data.getData();

                //Open file and read from it
                try {
                    if (android.os.Build.VERSION.SDK_INT >= android.os.Build.VERSION_CODES.KITKAT) {
                        String text = readTextFromUri(uri);

                        TaskUtil taskUtil = new TaskUtil(this);
                        taskUtil.loadBackup(text);

                        Toast.makeText(this, "Backup file loaded!", Toast.LENGTH_SHORT).show();
                    }
                }catch (Exception e){
                    Toast.makeText(this, "Error reading file!", Toast.LENGTH_SHORT).show();
                }
            }
        }

        super.onActivityResult(requestCode, resultCode, data);
    }

    @Override
    public void onBackPressed() {
        //Close activity
        this.finish();
    }




    @RequiresApi(api = Build.VERSION_CODES.KITKAT)
    private String readTextFromUri(Uri uri) throws IOException {
        StringBuilder stringBuilder = new StringBuilder();
        try (InputStream inputStream =
                     getContentResolver().openInputStream(uri);
             BufferedReader reader = new BufferedReader(
                     new InputStreamReader(Objects.requireNonNull(inputStream)))) {
            String line;
            while ((line = reader.readLine()) != null) {
                stringBuilder.append(line + "\n");
            }
        }
        return stringBuilder.toString();
    }

    private void writeDocument(Uri uri, String text) {
        try {
            ParcelFileDescriptor pfd = getContentResolver().
                    openFileDescriptor(uri, "w");
            FileOutputStream fileOutputStream =
                    new FileOutputStream(pfd.getFileDescriptor());
            fileOutputStream.write(text.getBytes());
            fileOutputStream.close();
            pfd.close();
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }


}
