/*
 * Copyright (C) 2021 Thomas Van Acker
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <https://www.gnu.org/licenses/>.
 */

package be.bitscuit.calendar.Util;

import android.app.AlarmManager;
import android.app.PendingIntent;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;

import java.util.ArrayList;
import java.util.Arrays;

import be.bitscuit.calendar.BroadcastReceivers.AlarmReceiver;

import static android.content.Context.ALARM_SERVICE;


public class TaskUtil {

    private Context context;
    private UserPrefs userPrefs;

    private String taskDataRaw;
    private TaskData[] taskData;
    private int nextTaskID;
    private int nextBatchID;

    public TaskUtil(Context context){
        this.context = context;
        userPrefs = new UserPrefs(context);
        reload();
    }


    public void reload(){
        System.out.println("Reload");
        SharedPreferences prefs = context.getSharedPreferences("TASK_DATA", Context.MODE_PRIVATE);
        taskDataRaw = prefs.getString("TASK_DATA_RAW", "");
        nextTaskID = prefs.getInt("NEXT_TASK_ID", 1000);
        nextBatchID = prefs.getInt("NEXT_BATCH_ID", 1000);

        //Parse data
        String[] lines = taskDataRaw.split("\n");
        ArrayList<TaskData> tasks = new ArrayList<>();
        boolean saveAfterParsing = false;//Whether to save the parsed array of tasks (used when removed old task)
        boolean deleteOldTasks = userPrefs.getAutoDeleteTasks();
        long deleteOldTasksAfter = userPrefs.getAutoDeleteTasksAfter();
        for(int i=0; i<lines.length; i++){
            try {
                TaskData td = fromString(lines[i]);

                //Check if can be deleted
                if(deleteOldTasks && td.getTimeEnd() < System.currentTimeMillis() - deleteOldTasksAfter) saveAfterParsing = true;
                else tasks.add(td);
            }catch (Exception e){}
        }

        //Sort tasks
        TaskData[] t = new TaskData[tasks.size()];
        t = tasks.toArray(t);
        //taskData = sortTaskData(t);
        taskData = t;//Don't need to sort


        //Save if needed
        if(saveAfterParsing){
            //Convert to raw
            taskDataRaw = "";
            for(int i=0; i<taskData.length; i++){
                taskDataRaw += toString(taskData[i]) + "\n";
            }

            //Save to prefs
            SharedPreferences.Editor editor = prefs.edit();
            editor.putString("TASK_DATA_RAW", taskDataRaw);
            editor.commit();//Must wait until saved

            reload();//Won't loop forever, because didn't save old tasks
        }

    }


    public TaskData[] getTaskDataForDate(long dateMillis){

        ArrayList<TaskData> tasks = new ArrayList<>();
        for(int i=0; i<taskData.length; i++){
            if(taskData[i].getTimeStart() >= dateMillis && taskData[i].getTimeStart() < dateMillis + 86400000){
                tasks.add(taskData[i]);
            }
        }

        TaskData[] ret = new TaskData[tasks.size()];
        ret = tasks.toArray(ret);
        //return sortTaskData(ret);//Don't need to sort
        return ret;
    }

    public TaskData getByTaskID(int taskID){
        for(int i=0; i<taskData.length; i++){
            if(taskData[i].getTaskID() == taskID){
                return taskData[i];
            }
        }
        return null;
    }

    public TaskData[] getByBatchID(int batchID){
        ArrayList<TaskData> t = new ArrayList<>();
        for(int i=0; i<taskData.length; i++){
            if(taskData[i].getBatchID() == batchID){
                t.add(taskData[i]);
            }
        }
        TaskData[] ret = new TaskData[t.size()];
        ret = t.toArray(ret);
        return ret;
    }

    public TaskData[] getAllTasks(){
        return taskData;
    }


    public static TaskData[] sortTaskData(TaskData[] tasks){
        TaskData[] sorted = new TaskData[tasks.length];
        String s = "";
        for(int i=0; i<tasks.length; i++){
            TaskData next = null;

            for(int j=0; j<tasks.length; j++){
                if(!s.contains(";"+tasks[j].getTaskID()+";")){
                    if(next == null){
                        next = tasks[j];
                    }else if(lessThen(tasks[j], next)){
                        next = tasks[j];
                    }
                }
            }

            sorted[i] = next;
            s += ";"+next.getTaskID()+";";
        }
        return sorted;
    }

    public static boolean lessThen(TaskData td0, TaskData td1){
        return td0.getTimeStart() < td1.getTimeStart() ||
                (td0.getTimeStart() == td1.getTimeStart() && td0.getTimeEnd() > td1.getTimeEnd());
    }


    public static TaskData fromString(String text){
        TaskData taskData = new TaskData();
        String[] slices = text.split(";");
        taskData.setTaskID(Integer.parseInt(slices[0]));
        taskData.setTimeStart(Long.parseLong(slices[1]));
        taskData.setTimeEnd(Long.parseLong(slices[2]));
        taskData.setTitle(slices[3]);
        taskData.setGroupID(Integer.parseInt(slices[4]));
        taskData.setBatchID(Integer.parseInt(slices[5]));
        taskData.setTimeNotify(Long.parseLong(slices[6]));
        taskData.setDescription(slices[7].equals("EMPTY")?"":slices[7]);
        return taskData;
    }

    public static String toString(TaskData task){
        String r = "";
        r += task.getTaskID()+";";
        r += task.getTimeStart()+";";
        r += task.getTimeEnd()+";";
        r += task.getTitle()+";";
        r += task.getGroupID()+";";
        r += task.getBatchID()+";";
        r += task.getTimeNotify()+";";
        r += (task.getDescription().equals("")?"EMPTY":task.getDescription())+";";
        return r;
    }


    public int getNextTaskID() {
        int ret = Integer.valueOf(nextTaskID);
        nextTaskID++;

        //Save new taskID
        SharedPreferences prefs = context.getSharedPreferences("TASK_DATA", Context.MODE_PRIVATE);
        SharedPreferences.Editor editor = prefs.edit();
        editor.putInt("NEXT_TASK_ID", nextTaskID);
        editor.apply();

        return ret;
    }

    public int getNextBatchID() {
        int ret = Integer.valueOf(nextBatchID);
        nextBatchID++;

        //Save new batchID
        SharedPreferences prefs = context.getSharedPreferences("TASK_DATA", Context.MODE_PRIVATE);
        SharedPreferences.Editor editor = prefs.edit();
        editor.putInt("NEXT_BATCH_ID", nextBatchID);
        editor.apply();

        return ret;
    }


    public void saveTask(TaskData task){
        //Adds task if new taskID or updates existing one
        for(int i=0; i<taskData.length; i++){
            if(task.getTaskID() == taskData[i].getTaskID()){
                //Remove old task
                removeTask(task.getTaskID());

                break;
            }
        }

        //Add task to taskData[]
        boolean added = false;
        TaskData[] t = new TaskData[taskData.length+1];
        for(int i=0; i<t.length; i++){
            if(added) t[i] = taskData[i-1];
            else if(i == t.length -1 || lessThen(task, taskData[i])){
                t[i] = task;
                added = true;
            }else t[i] = taskData[i];
        }
        taskData = t;


        //Convert to raw
        taskDataRaw = "";
        for(int i=0; i<taskData.length; i++){
            taskDataRaw += toString(taskData[i]) + "\n";
        }

        //Save to prefs
        SharedPreferences prefs = context.getSharedPreferences("TASK_DATA", Context.MODE_PRIVATE);
        SharedPreferences.Editor editor = prefs.edit();
        editor.putString("TASK_DATA_RAW", taskDataRaw);
        editor.apply();


        //Set new alarm for notification
        setAlarmForTask(task);


        //Reload
        reload();
    }

    public void removeTask(int taskID){
        //Generate new raw
        taskDataRaw = "";
        for(int i=0; i<taskData.length; i++){
            if(taskData[i].getTaskID() != taskID){
                taskDataRaw += toString(taskData[i]) + "\n";
            }
        }

        //Delete old alarm
        removeAlarmForTask(getByTaskID(taskID));

        //Save raw
        SharedPreferences prefs = context.getSharedPreferences("TASK_DATA", Context.MODE_PRIVATE);
        SharedPreferences.Editor editor = prefs.edit();
        editor.putString("TASK_DATA_RAW", taskDataRaw);
        editor.commit();//Must wait until save is done, so don't use apply()

        //Reload
        reload();
    }


    public void setAlarmForTask(TaskData task){
        if(task.getTimeNotify() == 0) return;
        AlarmManager alarmManager = (AlarmManager) context.getSystemService(ALARM_SERVICE);
        Intent i = new Intent(context, AlarmReceiver.class);
        i.putExtra("TASK_ID", task.getTaskID());
        PendingIntent pendingIntent = PendingIntent.getBroadcast(context, 0, i, 0);
        alarmManager.set(AlarmManager.RTC_WAKEUP, task.getTimeNotify(), pendingIntent);
    }

    public void removeAlarmForTask(TaskData task){
        if(task.getTimeNotify() == 0) return;
        AlarmManager alarmManager = (AlarmManager) context.getSystemService(ALARM_SERVICE);
        Intent i = new Intent(context, AlarmReceiver.class);
        i.putExtra("TASK_ID", task.getTaskID());
        PendingIntent pendingIntent = PendingIntent.getBroadcast(context, 0, i, PendingIntent.FLAG_UPDATE_CURRENT);
        pendingIntent.cancel();
        alarmManager.cancel(pendingIntent);
    }


    public void loadBackup(String text){
        String[] lines = text.split("\n");

        //First line -> special data stuff
        String[] slices = lines[0].split(";");
        nextTaskID = Math.max(Integer.parseInt(slices[0]), nextTaskID);
        nextBatchID = Math.max(Integer.parseInt(slices[1]), nextBatchID);
        getNextTaskID();//Save it
        getNextBatchID();//Save it

        //Set tasks
        for(int i=1; i<lines.length; i++){
            try {
                TaskData t = fromString(lines[i]);
                saveTask(t);
            }catch (Exception e){}
        }
    }

    public String getBackup(){
        String ret = "";
        ret += nextTaskID+";";
        ret += nextBatchID+";\n";

        for(int i=0; i<taskData.length; i++){
            ret += toString(taskData[i]) + "\n";
        }

        return ret;
    }

}
